@@ -2,24 +2,33 @@
  * Color lines for gnome
  * (c) 1999 Free Software Foundation
  * Authors: Robert Szokovacs <szo@appaloosacorp.hu>
- *          Shooby Ban <bansz@szif.hu>
+ *          Szabolcs Ban <shooby@gnome.hu>
  */
 
+
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
 #include <math.h>
+#include <dirent.h>
 
 #include <config.h>
 #include <gnome.h>
+
 #include <gdk_imlib.h>
+
 #include <libgnomeui/gnome-window-icon.h>
 
 #include "glines.h"
 
+#undef VERSION
+#define VERSION "1.9.0"
+
+GtkWidget *bah_window = NULL;
+
 GtkWidget *draw_area;
-static GtkWidget *app, *vb, *appbar;
+static GtkWidget *app, *vb, *appbar, *pref_dialog;
 GtkWidget *next_draw_area; /* XXX Shouldn't be this much externls! */
 GdkPixmap *backpixmap = NULL;
 GdkPixmap *next_backpixmap = NULL;
@@ -32,10 +41,80 @@ int active = -1;
 int target = -1;
 int inmove = 0;
 int score = 0;
+int ask_me = 0;
+int move_timeout = 100;
 int preview[3];
+char * ball_filename;
+char * box_filename;
 GtkWidget *scorelabel;
 scoretable sctab[] = {{5, 10}, {6, 12}, {7, 18}, {8, 28}, {9, 42}, {10, 82}, {11, 108}, {12, 138}, {13, 172}, {14, 210}, {0,0}};
 
+static void 
+load_image (gchar *fname,
+GdkPixmap **pixmap,
+GdkPixmap **mask)
+{
+	char *tmp, *fn;
+        GdkColor bgcolor;
+	GdkImlibImage *image;
+        GdkImage *tmpimage;
+	GdkVisual *visual;
+    
+	tmp = g_strconcat ( "glines/", fname, NULL);
+
+	fn = gnome_unconditional_pixmap_file ( tmp );
+	g_free( tmp );
+
+	if (!g_file_exists (fn)) {
+		printf (_("Could not find the \'%s\' theme for Glines\n"), fn);
+		exit (1);
+	}
+
+	if (!image)
+		gdk_imlib_destroy_image (image);
+
+	image = gdk_imlib_load_image (fn);
+	g_free( fn );
+
+	visual = gdk_imlib_get_visual ();
+	gdk_imlib_render (image, image->rgb_width, image->rgb_height);
+
+	if (!*pixmap)
+		gdk_imlib_free_pixmap (*pixmap);
+	if (!*mask)
+		gdk_imlib_free_pixmap (*mask);
+
+	*pixmap = gdk_imlib_move_image (image);
+        *mask = gdk_imlib_move_mask (image);
+
+	gdk_imlib_destroy_image (image);
+}
+
+static void
+help_cb (GtkWidget * widget, gpointer data)
+{
+
+
+  GnomeHelpMenuEntry help_entry = { "glines", "user-guide/usage.html#PREFERENCES-DIALOGBOX" };
+
+  gnome_help_display (NULL, &help_entry);
+}
+
+void
+set_inmove(int i)
+{
+ 	static int timeou = -1;
+ 	int ms;
+ 	
+ 	if (inmove != i) {
+ 	        inmove = i;
+ 	        ms = (inmove?move_timeout:100);
+ 	        if(timeou != -1) gtk_timeout_remove(timeou);
+ 	        timeou = gtk_timeout_add(ms, animate, (gpointer)draw_area);
+ 	}
+}
+
+
 void
 reset_game(void)
 {
@@ -56,22 +135,21 @@ reset_game(void)
 void
 start_game(void)
 {
-	static int timeou = -1;
 	char string[20];
 
-	draw_field(draw_area);
+	draw_field(draw_area,0);
 	draw_all_balls(draw_area, -1);
 	gtk_widget_draw(draw_area, NULL);
 	draw_preview();	
 	active = -1;
 	target = -1;
-	inmove = 0;
+	inmove = -1;
 	snprintf(string, 19, "%d", score);
 	gtk_label_set_text(GTK_LABEL(scorelabel), string);
-	if(timeou != -1) gtk_timeout_remove(timeou);
-	timeou = gtk_timeout_add(100, animate, (gpointer)draw_area);
+	set_inmove(0);
 }
 
+ 
 void
 reset_pathsearch(void)
 {
@@ -292,9 +370,11 @@ button_press_event (GtkWidget *widget, GdkEvent *event)
 
 	/* XXX Ezt megkapja */
 
+	if(inmove) return TRUE;
 	gtk_widget_get_pointer (widget, &x, &y);
 	fx = x / BOXSIZE;
 	fy = y / BOXSIZE;
+        gnome_appbar_set_status(GNOME_APPBAR(appbar), "");
 	if(field[fx + fy*9].color == 0)
 	{
 		/* Clicked on an empty field */
@@ -310,12 +390,12 @@ button_press_event (GtkWidget *widget, GdkEvent *event)
 			{
 				/* We found a route to the new position */
 
-				inmove = 1;
+				set_inmove(1);
 			}
 			else
 			{
-				/* Can't go there! */
-				gnome_appbar_set_status(GNOME_APPBAR(appbar), _("Can't go there!"));
+				/* Can't move there! */
+				gnome_appbar_set_status(GNOME_APPBAR(appbar), _("Can't move there!"));
 				reset_pathsearch();
 				target = -1;
 			}
@@ -353,14 +433,14 @@ button_press_event (GtkWidget *widget, GdkEvent *event)
 }
 
 void
-draw_field(GtkWidget *widget)
+draw_field(GtkWidget *widget, gint redraw)
 {
 	int i, j;
 
 	for(i=0; i<9; i++)
 		for(j=0; j<9; j++)
 		{
-			draw_box(widget, backpixmap, i, j, 0);
+			draw_box(widget, backpixmap, i, j, redraw);
 		}
 }
 
@@ -547,7 +627,7 @@ animate(gpointer gp)
 			newactive = active + 9;
 		else
 		{
-			inmove = 0;
+			set_inmove (0);
 		}
 		draw_box(widget, backpixmap, x, y, 1);
 		x = newactive%9;
@@ -560,7 +640,7 @@ animate(gpointer gp)
 		if(newactive == target)
 		{
 			target = -1;
-			inmove = 0;
+			set_inmove (0);
 			active = -1;
 			field[newactive].phase = 0;
 			field[newactive].active = 0;
@@ -619,38 +699,262 @@ game_about_callback (GtkWidget *widget, void *data)
 {
     GtkWidget *about;
     const gchar *authors[] = {
-		            "Robert Szokovacs <szo@appaloosacorp.hu>",
-			    "Szabolcs Ban <bansz@szif.hu>",
+		            _("Robert Szokovacs <szo@appaloosacorp.hu>"),
+			    _("Szabolcs Ban <shooby@gnome.hu>"),
 			    NULL
 			    };
 					        
 	about = gnome_about_new (_("Glines"), VERSION,
-			"(C) 1997-1998 the Free Software Foundation",
+			_("(C) 1997-2000 the Free Software Foundation"),
 			  (const char **)authors,
 		       _("Gnome port of the once-popular Color Lines game"),
-			"glines.xpm");
+			"glines.png");
 	gnome_dialog_set_parent(GNOME_DIALOG(about), GTK_WINDOW(app));
 	gtk_widget_show (about);
 	return TRUE;
 }       
 
 
+static void
+load_theme ()
+{
+	load_image(box_filename, &box_pixmap, &box_mask);
+	load_image(ball_filename, &ball_pixmap, &ball_mask);
+}
+
+static void
+glines_cancel (GtkWidget *widget, void *data)
+{
+	gtk_widget_destroy (pref_dialog);
+	pref_dialog = 0;
+}
+
+static void 
+load_theme_cb()
+{
+  gnome_config_set_string ("/glines/Table/BallTheme",
+                                     ball_filename);
+  gnome_config_set_string ("/glines/Table/BoxTheme",
+                                     box_filename);
+  gnome_config_set_int ("/glines/Prefs/AskMe",
+                                     ask_me);
+  gnome_config_set_int ("/glines/Prefs/MoveTimeout",
+                                     move_timeout);
+  gnome_config_sync();
+
+  load_theme();
+  draw_field(draw_area,1);
+  draw_all_balls(draw_area, -1);
+  draw_preview();
+  glines_cancel(0,0);
+}
+
+
+static void
+set_selection (GtkWidget *widget, void *data)
+{
+	if(!ball_filename)
+		g_free(ball_filename);
+	ball_filename = strdup(data);
+	if(!data)
+		free (data);
+}
+
+static void
+set_selection1 (GtkWidget *widget, void *data)
+{
+        if(!box_filename)
+		g_free(box_filename);
+	box_filename = strdup(data);
+	if(!data)
+		free (data);
+}
+
+static void
+free_str (GtkWidget *widget, void *data)
+{
+	if(!data)
+		free (data);
+}
+
+static void
+fill_menu (GtkWidget *menu, char * mtype, gboolean bg)
+{
+	struct dirent *e;
+	char *dname = gnome_unconditional_pixmap_file ("glines");
+	DIR *dir;
+        int itemno = 0;
+	
+	dir = opendir (dname);
+
+	if (!dir)
+		return;
+	
+	while ((e = readdir (dir)) != NULL){
+		GtkWidget *item;
+		char *s = strdup (e->d_name);
+
+		if (!strstr (e->d_name, mtype)) {
+			free (s);
+			continue;
+		}
+			
+		item = gtk_menu_item_new_with_label (s);
+		gtk_widget_show (GTK_WIDGET(item));
+		gtk_menu_append (GTK_MENU(menu), GTK_WIDGET(item));
+		if(bg)
+		gtk_signal_connect (GTK_OBJECT(item), "activate",
+				    GTK_SIGNAL_FUNC (set_selection1), s);
+		else
+		gtk_signal_connect (GTK_OBJECT(item), "activate",
+				    GTK_SIGNAL_FUNC (set_selection), s);
+		gtk_signal_connect (GTK_OBJECT(item), "destroy",
+				    GTK_SIGNAL_FUNC (free_str), s);
+	  
+	        if (bg){
+	        if (!strcmp(box_filename, s))
+		  gtk_menu_set_active(GTK_MENU(menu), itemno);
+		}
+	        else {
+	        if (!strcmp(ball_filename, s))
+		  gtk_menu_set_active(GTK_MENU(menu), itemno);
+		}
+			  
+	        itemno++;
+	}
+	closedir (dir);
+}
+
+static void
+set_selection_def (GtkWidget *widget, gpointer *data)
+{
+  ask_me = GTK_TOGGLE_BUTTON (widget)->active;
+}
+
+static void
+set_fast_moves (GtkWidget *widget, gpointer *data)
+{
+ 	move_timeout = GTK_TOGGLE_BUTTON (widget)->active ? 10 : 100;
+}
+
+game_props_callback (GtkWidget *widget, void *data)
+{
+	GtkWidget *menu1, *omenu1, *menu, *omenu, *l, *hb, *hb1, *f, *fv, *cb;
+	GtkWidget *button;
+
+	if (pref_dialog)
+		return;
+	
+	pref_dialog = gnome_dialog_new (_("Preferences"),
+			GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL,
+			GNOME_STOCK_BUTTON_HELP,
+			NULL);
+        gnome_dialog_set_default(GNOME_DIALOG (pref_dialog), 0);
+	gnome_dialog_set_parent (GNOME_DIALOG (pref_dialog), GTK_WINDOW (app));
+	gtk_signal_connect (GTK_OBJECT(pref_dialog), "delete_event",
+			    GTK_SIGNAL_FUNC (glines_cancel), NULL);
+
+	omenu = gtk_option_menu_new ();
+	menu = gtk_menu_new ();
+	fill_menu (menu,".png",FALSE);
+	gtk_widget_show (omenu);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU(omenu), menu);
+
+	omenu1 = gtk_option_menu_new ();
+	menu1 = gtk_menu_new ();
+	fill_menu (menu1,".xpm",TRUE);
+	gtk_widget_show (omenu1);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU(omenu1), menu1);
+
+	f = gtk_frame_new (_ ("Theme"));
+	gtk_container_border_width (GTK_CONTAINER (f), 5);
+
+	hb = gtk_hbox_new (FALSE, FALSE);
+	gtk_widget_show (hb);
+	
+	l = gtk_label_new (_("Select theme:"));
+	gtk_widget_show (l);
+	    
+	gtk_box_pack_start_defaults (GTK_BOX(hb), l);
+	gtk_container_add (GTK_CONTAINER(hb), omenu);
+
+	fv = gtk_vbox_new (0, 5);
+	gtk_container_border_width (GTK_CONTAINER (fv), 5);
+	gtk_widget_show (fv);
+	
+	gtk_box_pack_start_defaults (GTK_BOX(fv), hb);
+	hb1 = gtk_hbox_new (FALSE, FALSE);
+	gtk_widget_show (hb1);
+
+	l = gtk_label_new (_("Select background:"));
+	gtk_widget_show (l);
+	    
+	gtk_box_pack_start_defaults (GTK_BOX(hb1), l);
+	gtk_container_add (GTK_CONTAINER(hb1), omenu1);
+
+
+	gtk_container_add (GTK_CONTAINER(fv), hb1);
+
+	cb = gtk_check_button_new_with_label ( _("Fast moves") );
+	if (move_timeout == 10) {
+	        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb), TRUE);
+	}
+	gtk_signal_connect (GTK_OBJECT(cb), "clicked", (GtkSignalFunc)set_fast_moves, NULL);
+        gtk_widget_show (cb);
+
+	gtk_container_add (GTK_CONTAINER(fv), cb);
+
+	cb = gtk_check_button_new_with_label ( _("Ask confirmation when quitting") );
+	if (ask_me) 
+	 {
+	   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb),
+	                                   TRUE);
+	 }
+        gtk_signal_connect (GTK_OBJECT(cb), "clicked", (GtkSignalFunc)set_selection_def, NULL);
+	gtk_widget_show (cb);
+
+	gtk_container_add (GTK_CONTAINER(fv), cb);
+
+	gtk_box_pack_start_defaults (GTK_BOX(GNOME_DIALOG(pref_dialog)->vbox), f);
+	gtk_container_add (GTK_CONTAINER (f), fv);
+	
+	gtk_widget_show (f);
+	
+	gnome_dialog_button_connect (GNOME_DIALOG (pref_dialog), 0,
+			GTK_SIGNAL_FUNC (load_theme_cb), NULL);
+	gnome_dialog_button_connect (GNOME_DIALOG (pref_dialog), 1,
+			GTK_SIGNAL_FUNC (glines_cancel), (gpointer)1);
+	gnome_dialog_button_connect (GNOME_DIALOG (pref_dialog), 2,
+			GTK_SIGNAL_FUNC (help_cb), NULL);
+
+        gtk_widget_show (pref_dialog);
+}
+
 static int
 game_quit_callback (GtkWidget *widget, void *data)
 {
-		GtkWidget *box;
+	GtkWidget *box;
 
-	box = gnome_message_box_new (_("Do you really want to quit?"),
+	if (ask_me) {
+
+	 box = gnome_message_box_new (_("Do you really want to quit?"),
 	                             GNOME_MESSAGE_BOX_QUESTION,
 	                             GNOME_STOCK_BUTTON_YES,
 	                             GNOME_STOCK_BUTTON_NO,
 	                             NULL);
-	gnome_dialog_set_parent (GNOME_DIALOG(box), GTK_WINDOW(app));
-	gnome_dialog_set_default (GNOME_DIALOG (box), 0);
-	gtk_window_set_modal (GTK_WINDOW (box), TRUE);  
-	gtk_signal_connect (GTK_OBJECT (box), "clicked",
+	 gnome_dialog_set_parent (GNOME_DIALOG(box), GTK_WINDOW(app));
+	 gnome_dialog_set_default (GNOME_DIALOG (box), 0);
+	 gtk_window_set_modal (GTK_WINDOW (box), TRUE);  
+	 gtk_signal_connect (GTK_OBJECT (box), "clicked",
 	                   (GtkSignalFunc)game_maybe_quit, NULL);
-	gtk_widget_show (box);
+	 gtk_widget_show (box);
+
+	} 
+	else 
+	{ 
+	 gtk_widget_destroy (app);
+	 gtk_main_quit ();                               
+	}
 
  	return TRUE;
 
@@ -672,6 +976,10 @@ save_state (GnomeClient *client,
 
 	gnome_config_push_prefix (prefix);
 	gnome_config_set_int ("Glines/Score", score);
+
+	gnome_config_set_string ("Glines/BallTheme", ball_filename);
+
+	gnome_config_set_string ("Glines/BoxTheme", box_filename);
 	
 	buf = g_malloc(FIELDSIZE * FIELDSIZE * 4 + 1);
 	for(i = 0; i < FIELDSIZE * FIELDSIZE; i++)
@@ -692,12 +1000,31 @@ save_state (GnomeClient *client,
 	gnome_config_pop_prefix ();
 	gnome_config_sync();
 
-	argv[2]= gnome_config_get_real_path (prefix);
+/*	argv[2]= gnome_config_get_real_path (prefix);
 	gnome_client_set_discard_command (client, 3, argv);
-
+*/
 	return TRUE;
 }
 
+static void
+load_properties ()
+{
+  char buf[256];
+
+  g_snprintf (buf, 256, "/glines/Table/BallTheme=pulse.png");
+  ball_filename  = gnome_config_get_string (buf);
+
+  g_snprintf (buf, 256, "/glines/Table/BoxTheme=gray.xpm");
+  box_filename  = gnome_config_get_string (buf);
+
+  ask_me = gnome_config_get_int ("/glines/Prefs/AskMe=0");
+
+  move_timeout = gnome_config_get_int ("/glines/Prefs/MoveTimeout=100");
+
+  load_theme();
+}
+
+
 static void
 restart (void)
 {
@@ -752,15 +1079,23 @@ GnomeUIInfo gamemenu[] = {
 	GNOMEUIINFO_END
 };
 
-GnomeUIInfo helpmenu[] = {
-	GNOMEUIINFO_MENU_ABOUT_ITEM(game_about_callback, NULL),
+GnomeUIInfo settingsmenu[] = {
+        GNOMEUIINFO_MENU_PREFERENCES_ITEM(game_props_callback, NULL),
 
 	GNOMEUIINFO_END
 };
 
+
+GnomeUIInfo helpmenu[] = {
+        GNOMEUIINFO_HELP("glines"),
+	GNOMEUIINFO_MENU_ABOUT_ITEM(game_about_callback, NULL),
+	GNOMEUIINFO_END
+};
+
 GnomeUIInfo mainmenu[] = {
   	GNOMEUIINFO_MENU_GAME_TREE(gamemenu),
-		GNOMEUIINFO_MENU_HELP_TREE(helpmenu),
+	GNOMEUIINFO_MENU_SETTINGS_TREE(settingsmenu),
+	GNOMEUIINFO_MENU_HELP_TREE(helpmenu),
 	GNOMEUIINFO_END
 };
 
@@ -791,19 +1126,19 @@ main (int argc, char *argv [])
 	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/glines.xpm");
 	client = gnome_master_client ();
 
-	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",  
+	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
 	                    GTK_SIGNAL_FUNC (save_state), argv[0]);
 	gtk_signal_connect (GTK_OBJECT (client), "die",
                             GTK_SIGNAL_FUNC (client_die), NULL);
 
 	if (GNOME_CLIENT_RESTARTED (client))
-  {
+          {
 	    gnome_config_push_prefix(gnome_client_get_config_prefix (client));
 	    
 	    restart();  
 	    
 	    gnome_config_pop_prefix ();
-  }
+          }
 	else
 	{
 		reset_game();
@@ -856,8 +1191,8 @@ main (int argc, char *argv [])
 	gtk_widget_set_events (draw_area, gtk_widget_get_events(draw_area) |GDK_BUTTON_PRESS_MASK);
 
 	gtk_widget_realize(draw_area);
-	box_pixmap  = gdk_pixmap_create_from_xpm(draw_area->window, &box_mask,  NULL, gnome_unconditional_pixmap_file("glines/box.xpm"));
-	ball_pixmap = gdk_pixmap_create_from_xpm(draw_area->window, &ball_mask, NULL, gnome_unconditional_pixmap_file("glines/ball.xpm"));
+
+	load_properties(client);
 
 	backpixmap = gdk_pixmap_new(draw_area->window,
 			BOXSIZE*FIELDSIZE,
