@@ -310,6 +310,7 @@ Tetris::doSetup(GtkWidget *widget, void *d)
 	gnome_config_sync();
 	
 	t->scoreFrame->setLevel(t->startingLevel);
+	t->scoreFrame->setStartingLevel(t->startingLevel);
 	setupdialogDestroy(widget, d);
 	t->setupPixmap();
 }
@@ -666,8 +667,13 @@ Tetris::timeoutHandler(void *d)
 		if (res)
 		{
 			t->manageFallen();
-			t->fastFall = false;
+			if(t->fastFall)
+				t->scoreFrame->incScore(t->fastFallPoints);
+			else
+				t->fastFall = false;
 		}
+		else if(t->fastFall)
+			++t->fastFallPoints;
 	}
 
 	return 1;	
@@ -718,6 +724,7 @@ Tetris::eventHandler(GtkWidget *widget, GdkEvent *event, void *d)
 			if (!t->fastFall)
 			{
 				t->fastFall = true;
+				t->fastFallPoints = 0;
 				gtk_timeout_remove(t->timeoutId);
 				t->timeoutId = gtk_timeout_add(10, timeoutHandler, t);
 				res = true;
@@ -819,6 +826,7 @@ Tetris::gameNew(GtkWidget *widget, void *d)
 	int level = t->cmdlineLevel ? t->cmdlineLevel :
 		gnome_config_get_int_with_default("/gnometris/Properties/StartingLevel=1", 0);
 	t->scoreFrame->setLevel(level);
+	t->scoreFrame->setStartingLevel(level);
 	random_block_colors = gnome_config_get_int_with_default(
 		"/gnometris/Properties/RandomBlockColors=0", 0) != 0;
 	do_preview = gnome_config_get_int_with_default(
@@ -854,11 +862,11 @@ Tetris::gameAbout(GtkWidget *widget, void *d)
 
 	const gchar *authors[] = {"J. Marcin Gorycki", 0};
 
-	about = gnome_about_new("Gnometris", TETRIS_VERSION, "(C) 1999 J. Marcin Gorycki", 
+	about = gnome_about_new("Gnometris", TETRIS_VERSION, "(C) 2000 J. Marcin Gorycki", 
 							(const char **)authors,
 							_("Written for my wife, Matylda\n"
 							  "Send comments and bug reports to: "
-							  "marcin.gorycki@intel.com"), 0);
+							  "janusz.gorycki@intel.com"), 0);
 	gnome_dialog_set_parent(GNOME_DIALOG(about), GTK_WINDOW(t->w));
 	gtk_window_set_modal(GTK_WINDOW(about), TRUE);
 
