@@ -559,6 +559,7 @@ GyahtzeeCreateMainWindow(void)
 int
 main (int argc, char *argv[])
 {
+        gchar *newname, *PrefLoc;
         gint i;
 
 	gnome_score_init(appID);
@@ -575,14 +576,18 @@ main (int argc, char *argv[])
         gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-gtali.png");
 	NumberOfComputers = gnome_config_get_int("/gtali/Preferences/NumberOfComputerOpponents=5");
 	NumberOfHumans = gnome_config_get_int("/gtali/Preferences/NumberOfHumanOpponents=1");
+ 	ExtraYahtzeeBonus = gnome_config_get_int("/gtali/Preferences/ExtraYahtzeeBonus=1");
+ 	ExtraYahtzeeBonusVal = gnome_config_get_int("/gtali/Preferences/ExtraYahtzeeBonusVal=50");
+ 	ExtraYahtzeeJoker = gnome_config_get_int("/gtali/Preferences/ExtraYahtzeeJoker=0");
 
         /* Read in new player names */
 	for (i = 0; i < MAX_NUMBER_OF_PLAYERS; ++i) {
                 gchar *newname;
-                gchar PrefLoc[] = "/gtali/Preferences/PlayerName1"; /* Careful renaming this */
-                
-                PrefLoc[29] = i+'1';
+
+                PrefLoc = g_strdup_printf("/gtali/Preferences/PlayerName%1d",i+1);
                 newname = gnome_config_get_string_with_default(PrefLoc, NULL);
+                g_free(PrefLoc);
+
                 if (newname) {
                         players[i].name = g_malloc(strlen(newname));
                         if (players[i].name)
